package org.eclipse.html.editor;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.presentation.IPresentationDamager;
import org.eclipse.jface.text.presentation.IPresentationRepairer;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.custom.StyleRange;

/**
 * Implementation of the damager and repairer interfaces for reconciling
 * damaged javadoc comments.
 */
public class CommentDamagerRepairer
   implements IPresentationDamager, IPresentationRepairer {
   /** 
    * The text attribute for HTML comments
    */
   protected TextAttribute attribute;
   /** 
    * The document this object works on 
    */
   protected IDocument document;

   /**
    * Creates a new CommentDamagerRepairer.
    */
   public CommentDamagerRepairer(TextAttribute attribute) {
      this.attribute = attribute;
   }
   /**
    * @see IPresentationRepairer#createPresentation(TextPresentation, ITypedRegion)
    */
   public void createPresentation(TextPresentation pr, ITypedRegion region) {
      pr.addStyleRange(
         new StyleRange(
            region.getOffset(),
            region.getLength(),
            attribute.getForeground(),
            attribute.getBackground(),
            attribute.getStyle()));
   }
   /**
    * Returns the end offset of the line that contains the specified offset or
    * if the offset is inside a line delimiter, the end offset of the next line.
    *
    * @param offset the offset whose line end offset must be computed
    * @return the line end offset for the given offset
    * @exception BadLocationException if offset is invalid in the current document
    */
   protected int endOfLineOf(int offset) throws BadLocationException {
      IRegion region = document.getLineInformationOfOffset(offset);
      int endRegion = region.getOffset() + region.getLength();
      if (offset <= endRegion)
         return endRegion;
      int line = document.getLineOfOffset(offset);
      try {
         region = document.getLineInformation(line + 1);
         return region.getOffset() + region.getLength();
      } catch (BadLocationException x) {
         return document.getLength();
      }
   }
   /**
    * @see IPresentationDamager#getDamageRegion(ITypedRegion, DocumentEvent, boolean)
    */
   public IRegion getDamageRegion(
      ITypedRegion partition,
      DocumentEvent event,
      boolean partioningChanged) {
      if (partioningChanged)
         return partition;
      try {
         IRegion info = document.getLineInformationOfOffset(event.getOffset());
         int start = Math.max(partition.getOffset(), info.getOffset());
         int end =
            event.getOffset()
               + (event.getText() == null
                  ? event.getLength()
                  : event.getText().length());
         if (info.getOffset() <= end
            && end <= info.getOffset() + info.getLength()) {
            // optimize the case of the same line
            end = info.getOffset() + info.getLength();
         } else
            end = endOfLineOf(end);
         end = Math.min(partition.getOffset() + partition.getLength(), end);
         return new Region(start, end - start);
      } catch (BadLocationException x) {
         return partition;
      }
   }
   /**
    * @see IPresentationRepairer#setDocument(IDocument)
    */
   public void setDocument(IDocument document) {
      this.document = document;
   }
}