package org.eclipse.html.editor;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.*;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.*;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

public class HTMLConfiguration extends SourceViewerConfiguration {
   /**
    * Partition type for HTML comments
    */
   public final static String HTML_COMMENT = "__html_comment";
   /**
    * Partition type of HTML tags
    */
   public final static String HTML_TAG = "__html_tag";

   /**
    * Colors for HTML editor
    */
   private Color colorComment, colorDefault, colorString, colorTag;

   private HTMLDoubleClickStrategy doubleClickStrategy;
   private RuleBasedScanner scanner;
   private RuleBasedScanner tagScanner;
   private IWhitespaceDetector whitespaceDetector = new IWhitespaceDetector() {
      public boolean isWhitespace(char c) {
         return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
      }
   };

   public HTMLConfiguration() {
      Display display = Display.getCurrent();
      colorComment = new Color(display, new RGB(128, 0, 0));
      colorDefault = new Color(display, new RGB(0, 0, 0));
      colorString = new Color(display, new RGB(0, 128, 0));
      colorTag = new Color(display, new RGB(0, 0, 128));
   }

   private IToken createToken(Color color) {
      return new Token(new TextAttribute(color));
   }
   public void dispose() {
      colorComment.dispose();
      colorDefault.dispose();
      colorString.dispose();
      colorTag.dispose();
   }
   public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
      return new String[] {
         IDocument.DEFAULT_CONTENT_TYPE,
         HTML_COMMENT,
         HTML_TAG };
   }
   public IContentAssistant getContentAssistant(ISourceViewer sv) {
      ContentAssistant ca = new ContentAssistant();
      IContentAssistProcessor pr = new TagCompletionProcessor();
      ca.setContentAssistProcessor(pr, HTML_TAG);
      ca.setContentAssistProcessor(pr,
         IDocument.DEFAULT_CONTENT_TYPE);
      ca.setInformationControlCreator(
         getInformationControlCreator(sv));
      return ca;
   }
   public ITextDoubleClickStrategy getDoubleClickStrategy(
      ISourceViewer sourceViewer,
      String contentType) {
      if (doubleClickStrategy == null)
         doubleClickStrategy = new HTMLDoubleClickStrategy();
      return doubleClickStrategy;
   }
   private RuleBasedScanner getBaseScanner() {
      if (scanner == null) {
         scanner = new RuleBasedScanner();
         scanner.setRules(new IRule[] {
            // generic whitespace rule.
            new WhitespaceRule(whitespaceDetector)});
         scanner.setDefaultReturnToken(createToken(colorDefault));
      }
      return scanner;
   }
   private RuleBasedScanner getTagScanner() {
      if (tagScanner == null) {
         tagScanner = new RuleBasedScanner();
         IToken string = createToken(colorString);
         IRule[] rules = new IRule[3];
         // Add rule for double quotes
         rules[0] = new SingleLineRule("\"", "\"", string, '\\');
         // Add a rule for single quotes
         rules[1] = new SingleLineRule("'", "'", string, '\\');
         // Add generic whitespace rule.
         rules[2] = new WhitespaceRule(whitespaceDetector);
         tagScanner.setRules(rules);
         tagScanner.setDefaultReturnToken(createToken(colorTag));
      }
      return tagScanner;
   }
   public IPresentationReconciler getPresentationReconciler(ISourceViewer sv) {
      PresentationReconciler rec = new PresentationReconciler();

      DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getTagScanner());
      rec.setDamager(dr, HTML_TAG);
      rec.setRepairer(dr, HTML_TAG);

      dr = new DefaultDamagerRepairer(getBaseScanner());
      rec.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
      rec.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

      CommentDamagerRepairer sdr =
         new CommentDamagerRepairer(new TextAttribute(colorComment));
      rec.setDamager(sdr, HTML_COMMENT);
      rec.setRepairer(sdr, HTML_COMMENT);
      return rec;
   }
}