package org.eclipse.html.editor;

import org.eclipse.jface.text.*;

/**
 * Implements the behaviour on double-click in the HTML editor example. First
 * tries to select the word under the mouse. If no word is found, the entire
 * HTML element is selected.
 */
public class HTMLDoubleClickStrategy implements ITextDoubleClickStrategy {
   /**
    * Sample snippet for FAQ 00093.html showing a trivial double click strategy
    * that selects the range up to the next vowel.
    */
   public void doubleClickedSampleSnippet(ITextViewer part) {
      try {
         final int offset = part.getSelectedRange().x;
         int length = 0;
         IDocument doc = part.getDocument();
         while (true) {
            char c = doc.getChar(offset + length);
            if (c == 'a' || c == 'e' || c == 'i' || c == 'o' || c == 'u')
               break;
            if (offset + ++length >= doc.getLength())
               return;
         }
         part.setSelectedRange(offset, length);
      } catch (BadLocationException e) {
         e.printStackTrace();
      }
   }
   public void doubleClicked(ITextViewer part) {
      int pos = part.getSelectedRange().x;
      if (pos < 0)
         return;
      if (!selectWord(part, pos))
         selectElement(part, pos);
   }
   /**
    * Selects the entire HTML element (body or tag) currently under the mouse.
    */
   protected boolean selectElement(ITextViewer part, int caret) {
      IDocument doc = part.getDocument();
      try {
         int start = caret;
         final int length = doc.getLength();
         int end = length;
         char c = ' ';
         while (start >= 0) {
            c = doc.getChar(start);
            if (c == '<') {
               end = doc.search(caret, ">", true, false, false);
               break;
            } else if (c == '>') {
               int beginTag = doc.search(start, "<", false, false, false);
               if (beginTag >= 0) {
                  String tag = "</" + doc.get(beginTag + 1, start - beginTag);
                  end = doc.search(caret, tag, true, true, false);
               }
               break;
            }
            --start;
         }
         if (end >= length || end < 0)
            return false;
         selectRange(part, start, end);
         return true;
      } catch (BadLocationException x) {
      }
      return false;
   }
   private void selectRange(ITextViewer part, int start, int end) {
      int offset = start + 1;
      int length = end - offset;
      part.setSelectedRange(offset, length);
   }
   /**
    * Selects the entire word under the mouse. Returns true if a word was
    * selected, and false if no word was found.
    */
   protected boolean selectWord(ITextViewer part, int caret) {
      IDocument doc = part.getDocument();
      try {
         int start = caret;
         char c;
         while (start >= 0) {
            c = doc.getChar(start);
            if (!Character.isLetterOrDigit(c))
               break;
            --start;
         }
         int end = caret;
         int length = doc.getLength();
         while (end < length) {
            c = doc.getChar(end);
            if (!Character.isLetterOrDigit(c))
               break;
            ++end;
         }
         if (start == end)
            return false;
         selectRange(part, start, end);
         return true;
      } catch (BadLocationException x) {
      }
      return false;
   }
}