package org.eclipse.html.editor;

import java.util.ArrayList;

import org.eclipse.jface.text.*;
import org.eclipse.jface.text.contentassist.*;

/**
 * Completion processor for code assist on HTML tags
 */
public class TagCompletionProcessor implements IContentAssistProcessor {
   private static final IContextInformation[] NO_CONTEXTS = new IContextInformation[0];
   private static final char[] PROPOSAL_ACTIVATION_CHARS = new char[] { '<' };
   private ICompletionProposal[] NO_COMPLETIONS = new ICompletionProposal[0];
   public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int offset) {
      IDocument document = viewer.getDocument();
      //try HTML tag completions
      ICompletionProposal[] completions = computeTagCompletions(document, offset);
      if (completions != null)
         return completions;

      //try escape sequence completions
      completions = computeEscapeCompletions(document, offset);
      if (completions != null)
         return completions;

      //no guesses
      return NO_COMPLETIONS;
   }
   public IContextInformation[] computeContextInformation(ITextViewer viewer, int documentOffset) {
      return NO_CONTEXTS;
   }
   /**
    * Computes completion proposals on HTML escape sequences.  Returns
    * all relevant completions.  Returns an empty array if the offset was
    * in the context of an escape sequence but there are no matches,
    * or null if the document was not in the context of an escape sequence.
    */
   private ICompletionProposal[] computeEscapeCompletions(IDocument document, int offset) {
      try {
         int escapeStart = lastIndexOf(document, '&', offset);
         if (escapeStart == -1)
            return null;
         String prefix = firstWord(document, escapeStart, offset);
         //if the first word is not part of the escape sequence, then we can't complete
         if (escapeStart + prefix.length() < offset)
            return null;
         return matchingCompletions(EscapeCompletion.allCompletions(), prefix, escapeStart-1);
      } catch (BadLocationException e) {
      }
      return null;
   }
   private ICompletionProposal[] computeTagCompletions(IDocument document, int offset) {
      try {
         int tagStart = lastIndexOf(document, '<', offset);
         if (tagStart < 0)
            return null;
         String tag = firstWord(document, tagStart, offset);
         if (tagStart + tag.length() >= offset) {
            //if the tag has no end, then return completions on the tag name
            return matchingCompletions(TagCompletion.allCompletions(), tag, tagStart - 1);
         }
         //compute completions on current tag attributre
         String prefix = lastWord(document, tagStart + tag.length(), offset - 1);
         return matchingCompletions(
            AttributeCompletion.attributesFor(tag),
            prefix,
            offset - prefix.length());
      } catch (BadLocationException e) {
         return null;
      }
   }
   /**
    * Returns the first complete word in the document between
    * the given offsets.  Returns the empty string if there is no such
    * word.
    */
   private String firstWord(IDocument doc, int start, int end) throws BadLocationException {
      for (int i = start; i < end; i++) {
         char c = doc.getChar(i);
         switch (c) {
            case ' ' :
            case '\t' :
            case '<' :
            case '>' :
               return doc.get(start, i - start);
         }
      }
      return doc.get(start, end - start);
   }
   public char[] getCompletionProposalAutoActivationCharacters() {
      return PROPOSAL_ACTIVATION_CHARS;
   }
   public char[] getContextInformationAutoActivationCharacters() {
      return null;
   }
   public IContextInformationValidator getContextInformationValidator() {
      return null;
   }
   public String getErrorMessage() {
      return null;
   }
   /**
    * Computes and returns the offset of the last index of the given character
    * before the given offset.  Returns -1 if the end of a tag is reached.
    */
   private int lastIndexOf(IDocument doc, char ch, int offset) throws BadLocationException {
      for (int i = offset - 1; i >= 0; i--) {
         char c = doc.getChar(i);
         if (c == ch)
            return i + 1;
         else if (c == '>')
            //no prefix found 
            return -1;
      }
      return -1;
   }
   /**
    * Returns the last completion word in the document between
    * the given offsets.  Returns the empty string if there is no such
    * word.
    */
   private String lastWord(IDocument doc, int start, int end) throws BadLocationException {
      for (int i = end; i >= start; i--) {
         char c = doc.getChar(i);
         switch (c) {
            case ' ' :
            case '\t' :
            case '<' :
            case '>' :
               return doc.get(i + 1, end - i);
         }
      }
      return "";
   }
   private ICompletionProposal[] matchingCompletions(
      HTMLCompletion[] completions,
      String prefix,
      int replaceOffset) {
      ArrayList matches = new ArrayList();
      for (int i = 0; i < completions.length; i++) {
         if (completions[i].isMatching(prefix))
            matches.add(
               new CompletionProposal(
                  completions[i].getReplaceText(),
                  replaceOffset,
                  prefix.length() + 1,
                  completions[i].getCursorOffset(),
                  null,
                  completions[i].getReplaceText(),
                  null,
                  completions[i].getDescription()));
      }
      return (ICompletionProposal[]) matches.toArray(new ICompletionProposal[matches.size()]);
   }
}